#
# spec file for package distribution
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define goipath github.com/distribution/distribution/v3
Name:           distribution
Version:        3.0.0
Release:        1.1
Summary:        The toolset to pack, ship, store, and deliver content
License:        Apache-2.0
Group:          System/Management
URL:            https://github.com/distribution/distribution
Source0:        distribution-%{version}.tar.zst
Source1:        registry-configuration.yml
Source2:        registry.service
Source4:        README-registry.SUSE
Source10:       system-user-registry.conf
BuildRequires:  golang-packaging
BuildRequires:  systemd-rpm-macros
BuildRequires:  sysuser-tools
BuildRequires:  zstd
BuildRequires:  golang(API) = 1.24
Provides:       docker-distribution = %{version}
Obsoletes:      docker-distribution < %{version}
ExclusiveArch:  %ix86 x86_64 %arm aarch64 ppc64 ppc64le s390x riscv64

%description
The toolset to pack, ship, store, and deliver content.

This repository's main product is the Open Source Registry implementation for
storing and distributing container images using the OCI Distribution
Specification. The goal of this project is to provide a simple, secure, and
scalable base for building a large scale registry solution or running a simple
private registry. It is a core library for many registry operators including
Docker Hub, GitHub Container Registry, GitLab Container Registry and
DigitalOcean Container Registry, as well as the CNCF Harbor Project, and VMware
Harbor Registry.  The Docker toolset to pack, ship, store, and deliver content.

%package registry
Summary:        Registry server following OCI Distribution Specification
Group:          System/Management
Provides:       docker-distribution-registry = %{version}
Obsoletes:      docker-distribution-registry < %{version}
# shell of the registry user we create on %%pre
Requires(pre):  /usr/sbin/nologin
%sysusers_requires
%{?systemd_ordering}

%description registry
Registry server for Docker (hosting/delivering of repositories and images).

%prep
%setup -q -n distribution-%{version}
cp %{SOURCE4} .

%build
%sysusers_generate_pre %{SOURCE10} registry system-user-registry.conf

%{goprep} %{goipath}
export CGO_ENABLED=1 GO111MODULE=auto
%define buildtags "include_oss include_gcs"
%define ldflags "-s -w -X %{goipath}/version.Version=v%{version} -X %{goipath}/version.Package=%{goipath}"

for cmd in registry digest registry-api-descriptor-template; do
    %{gobuild} -trimpath -ldflags %{ldflags} -tags %{buildtags} cmd/$cmd
done

%install
install -d -m755 %{buildroot}%{_sysusersdir}
install -m 0644 %{SOURCE10} %{buildroot}%{_sysusersdir}/

%{goinstall}
install -D -m644 %{SOURCE1} %{buildroot}/%{_sysconfdir}/registry/config.yml
install -d  %{buildroot}%{_localstatedir}/lib/docker-registry

#
# systemd service for registry
#
install -Dd -m 0755  %{buildroot}%{_sbindir}
install -D  -m 0644  %{SOURCE2} %{buildroot}%{_unitdir}/registry.service
ln -sv %{_sbindir}/service %{buildroot}%{_sbindir}/rcregistry

%pre registry -f registry.pre
%service_add_pre registry.service

%post registry
%service_add_post registry.service

%preun registry
%service_del_preun registry.service

%postun registry
%service_del_postun registry.service

%files registry
%license LICENSE
%doc README.md README-registry.SUSE
%{_bindir}/digest
%{_bindir}/registry
%{_bindir}/registry-api-descriptor-template
%{_sbindir}/rcregistry
%{_unitdir}/registry.service
%{_sysusersdir}/system-user-registry.conf
%config %{_sysconfdir}/registry
%config(noreplace) %{_sysconfdir}/registry/config.yml
%attr(-,registry,registry) %{_localstatedir}/lib/docker-registry

%changelog
* Mon Jun 30 2025 Dirk Müller <dmueller@suse.com>
- update to 3.0.0 (jsc#PED-11728):
  * This is the first v3 stable release since `v2.8.3` which is a
    culmination of years of hard work of the container community
    and registry maintainers!
  * If you are upgrading from `v2.x` and have never used any of
    the release candidates, please familiarise yourselves with
    the `v2.x` deprecations properly.
  * oss and swift storage drivers are no longer supported
  * `docker/libtrust` has been replaced with `go-jose/go-jose` in
    https://github.com/distribution/distribution/pull/4096
  * `client` is no longer supported as a standalone package in
    https://github.com/distribution/distribution/pull/4126
  * the default configuration path has changed to
    `/etc/distribution/config.yml`
  * `ManifestBuilder` interface in 3886
  * `manifest.Versioned` has been deprecated in favor of
    `oci.Versioned` in 3887
  * `reference` package has been moved to
    github.com/distribution/reference in
    https://github.com/distribution/distribution/pull/4063
* Wed Oct 23 2024 Dirk Müller <dmueller@suse.com>
- remove unnecessary lock group
* Mon Nov 27 2023 Dirk Müller <dmueller@suse.com>
- update to 2.8.3 (bsc#1216491):
  * Pass `BUILDTAGS` argument to `go build`
  * Enable Go build tags
  * `reference`: replace deprecated function `SplitHostname`
  * Dont parse errors as JSON unless Content-Type is set to JSON
  * update to go 1.20.8
  * Set `Content-Type` header in registry client `ReadFrom`
  * deprecate reference package, migrate to
    github.com/distribution/reference
  * `digestset`: deprecate package in favor of `go-
    digest/digestset`
  * Do not close HTTP request body in HTTP handler
  * Add v2.8.3 release notes
* Fri May 12 2023 Dirk Müller <dmueller@suse.com>
- update to 2.8.2 (bsc#1207705, CVE-2023-2253, bsc#1210428):
  * Revert registry/client: set `Accept: identity` header when
    getting layers
  * Parse `http` forbidden as denied
  * Fix CVE-2023-2253 runaway allocation on /v2/_catalog
  * Fix panic in inmemory driver
  * update to go1.19.9
  * Add code to handle pagination of parts. Fixes max layer size
    of 10GB bug
  * Dockerfile: fix filenames of artifacts
- drop aws-sdk-1.42.27-update.patch (no longer wanted)
- drop 0001-Fix-runaway-allocation-on-v2-_catalog.patch (upstream)
* Wed Apr 26 2023 Dirk Müller <dmueller@suse.com>
- refresh 0001-Fix-runaway-allocation-on-v2-_catalog.patch to
  be more compatible with invalid pagination requests (CVE-2023-2253, bsc#1207705)␣
* Fri Feb 17 2023 Dirk Müller <dmueller@suse.com>
- add 0001-Fix-runaway-allocation-on-v2-_catalog.patch (CVE-2023-2253, bsc#1207705)
* Fri Jan 13 2023 Dirk Müller <dmueller@suse.com>
- add aws-sdk-1.42.27-update.patch to include AWS_ROLE_ARN
  support (SD-106252)
* Fri Sep  9 2022 Frederic Crozat <fcrozat@suse.com>
- Explicitly require nologin shell which is needed for registry
  system user (bsc#1203324)
* Wed Jun  8 2022 Dirk Müller <dmueller@suse.com>
- switch to go 1.16 for 2.8.1 (jsc#SLE-24963)
- build the additional commands as well
- rename to follow upstream renaming
- cleanups
- switch to systemd for user generation
* Mon May  2 2022 dcermak@suse.com
- Update to version 2.8.1:
  * FIx typo
  * Update 2.8.1. release notes
  * go 1.16.15
* Tue Feb 15 2022 rbrown@suse.com
- Update to version 2.8.0:
  * Update README so the release pipeline works properly.
  * Make this releaes a beta release first.
  * [2.8] Release artifacts
  * release workflow
  * Apply suggestions from code review
  * Prepare for v2.8.0 release
  * [release/2.7] vendor: github.com/opencontainers/image-spec v1.0.2
  * manifest: validate document type before unmarshal
  * [release/2.7] vendor: github.com/golang-jwt/jwt v3.2.1
- Drop merged patchs
  * 0001-Fix-s3-driver-for-supporting-ceph-radosgw.patch
  * 0002-Relax-filesystem-driver-folder-permissions-to-0777-cont.patch
  * 0003-Support-external-redis-sentinel-cluster.patch
  * 0004-Make-ipfilteredby-not-required.patch
* Thu Jul 15 2021 Dirk Müller <dmueller@suse.com>
- go back to golang 1.15 to fix the build for now
* Mon Oct 12 2020 Dirk Mueller <dmueller@suse.com>
- add 0004-Make-ipfilteredby-not-required.patch
  Make ipfilteredby optional
* Thu Sep 10 2020 Jiri Suchomel <jsuchome@suse.com>
- Add 0003-Support-external-redis-sentinel-cluster.patch
  This is patch that is not officially merged in docker-distribution,
  but is being used by Harbor project when building docker-distribution
  image
- Bumped go BuildRequires to 1.13
* Tue Aug 18 2020 Stefan Nica <snica@suse.com>
- Add 0002-Relax-filesystem-driver-folder-permissions-to-0777-cont.patch
  to get the registry to honor the umask when creating new folders
  (https://github.com/docker/distribution/pull/3204)
- Add 0001-Fix-s3-driver-for-supporting-ceph-radosgw.patch
  (https://github.com/docker/distribution/pull/2879)
* Fri Jul 31 2020 Stefan Nica <snica@suse.com>
- include OSS and GCS backend drivers in the build
* Tue Jul 28 2020 Thorsten Kukuk <kukuk@suse.com>
- Remove registry.SuSEfirewall2, SuSEfirewall2 does not exist anymore
* Fri Jul 24 2020 jsuchome@suse.com
- enabled changesgenerate option to automatically generate changes
* Tue Jul  7 2020 Stefan Nica <snica@suse.com>
- Run registry as non-privileged user
* Thu Jun 25 2020 Stefan Nica <snica@suse.com>
- Reworked the RPM spec to use the golang-packaging macros instead
  of the provided Makefile
* Fri Feb  7 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable build on %%arm (which include armv6), not only on armv7
* Wed Oct 30 2019 Michel Normand <normand@linux.vnet.ibm.com>
- Enable ppc64le
* Thu Oct  3 2019 kukuk@suse.de
- Use correct URL to project
- Remove fillup, we don't ship a sysconfig file
- Correct systemd requires
- Enable build on ARM
* Fri Jan 18 2019 msabate@suse.com
- Upgraded to 2.7.1
  - Support for OCI images added
  - Fix upgrade issues from 2.6.x
  - Update Go version to 1.11
  - Switch to multi-stage Dockerfile
  - Validations enabled by default with new disabled config option
  - Optimize health check performance
  - Create separate permission for deleting objects in a repo
  - Fix storage driver error propagation for manifest GETs
  - Fix forwarded header resolution
  - Add prometheus metrics
  - Disable schema1 manifest by default
  - Graceful shutdown
  - TLS: remove ciphers that do not support perfect forward secrecy
  - Fix registry stripping newlines from manifests
  - Add bugsnag logrus hook
  - Support ARM builds
* Wed Jun 13 2018 dcassany@suse.com
- Make use of %%license macro
* Thu Mar  1 2018 jmassaguerpla@suse.com
- Configuration files should be tagged in general as %%config(noreplace)
  in order to keep the modified config files.
  see https://en.opensuse.org/openSUSE:Specfile_guidelines#Configuration_files
  This fixes bsc#1083474
* Fri Jul 21 2017 msabate@suse.com
- Updated to 2.6.2
  This release is a special security release to address an issue allowing
  an attacker to force arbitrarily-sized memory allocations in a registry
  instance through the manifest endpoint. The problem has been mitigated
  by limiting the size of reads for image manifest content.
  Details for mitigation are in 29fa466
  Fixes bsc#1049850(CVE-2017-11468)
* Mon Jun 12 2017 abergmann@suse.com
- Add SuSEfirewall2 service file for TCP port 5000.
* Thu Apr  6 2017 msabate@suse.com
- Updated to 2.6.1;
  * Fix Forwarded header handling, revert use of X-Forwarded-Port
  * Use driver Stat for registry health check
  fix bsc#1033172
* Fri Jan 27 2017 bg@suse.com
- enable build for s390x
* Wed Jan 18 2017 msabate@suse.com
- Updated to v2.6.0
  [#]### Storage
- S3: fixed bug in delete due to read-after-write inconsistency
- S3: allow EC2 IAM roles to be used when authorizing region endpoints
- S3: add Object ACL Support
- S3: fix delete method's notion of subpaths
- S3: use multipart upload API in `Move` method for performance
- S3: add v2 signature signing for legacy S3 clones
- Swift: add simple heuristic to detect incomplete DLOs during read ops
- Swift: support different user and tenant domains
- Swift: bulk deletes in chunks
- Aliyun OSS: fix delete method's notion of subpaths
- Aliyun OSS: optimize data copy after upload finishes
- Azure: close leaking response body
- Fix storage drivers dropping non-EOF errors when listing repositories
- Compare path properly when listing repositories in catalog
- Add a foreign layer URL host whitelist
- Improve catalog enumerate runtime
  [#]### Registry
- Export `storage.CreateOptions` in top-level package
- Enable notifications to endpoints that use self-signed certificates
- Properly validate multi-URL foreign layers
- Add control over validation of URLs in pushed manifests
- Proxy mode: fix socket leak when pull is cancelled
- Tag service: properly handle error responses on HEAD request
- Support for custom authentication URL in proxying registry
- Add configuration option to disable access logging
- Add notification filtering by target media type
- Manifest: `References()` returns all children
- Honor `X-Forwarded-Port` and Forwarded headers
- Reference: Preserve tag and digest in With* functions
- Add policy configuration for enforcing repository classes
  [#]### Client
- Changes the client Tags `All()` method to follow links
- Allow registry clients to connect via HTTP2
- Better handling of OAuth errors in client
  [#]### Spec
- Manifest: clarify relationship between urls and foreign layers
- Authorization: add support for repository classes
  [#]### Manifest
- Override media type returned from `Stat()` for existing manifests
- Add plugin mediatype to distribution manifest
  [#]### Docs
- Document `TOOMANYREQUESTS` error code
- Document required Let's Encrypt port
- Improve documentation around implementation of OAuth2
- Improve documentation for configuration
  [#]### Auth
- Add support for registry type in scope
- Add support for using v2 ping challenges for v1
- Add leeway to JWT `nbf` and `exp` checking
- htpasswd: dynamically parse htpasswd file
- Fix missing auth headers with PATCH HTTP request when pushing to default port
  [#]### Dockerfile
- Update to go1.7
- Reorder Dockerfile steps for better layer caching
  [#]### Notes
  Documentation has moved to the documentation repository at
  `github.com/docker/docker.github.io/tree/master/registry`
  The registry is go 1.7 compliant, and passes newer, more restrictive `lint` and `vet` ing.
* Mon Aug  1 2016 msabate@suse.com
- Updated to v2.5.0
  [#]## Storage
- Ensure uploads directory is cleaned after upload is commited
- Add ability to cap concurrent operations in filesystem driver
- S3: Add 'us-gov-west-1' to the valid region list
- Swift: Handle ceph not returning Last-Modified header for HEAD requests
- Add redirect middleware
  [#]### Registry
- Add support for blobAccessController middleware
- Add support for layers from foreign sources
- Remove signature store
- Add support for Let's Encrypt
- Correct yaml key names in configuration
  [#]### Client
- Add option to get content digest from manifest get
  [#]### Spec
- Update the auth spec scope grammar to reflect the fact that hostnames are optionally supported
- Clarify API documentation around catalog fetch behavior
  [#]## API
- Support returning HTTP 429 (Too Many Requests)
  [#]## Documentation
- Update auth documentation examples to show "expires in" as int
  [#]## Docker Image
- Use Alpine Linux as base image
* Wed May 18 2016 msabate@suse.com
- Updated to v2.4.1. It contains the following fixes:
  93d7624 Preserve author information in schema1 manifests
  ba672e8 When a blob upload is committed prevent writing out hashstate in the subsequent close.
  96230de Add a test with a missing _manifests directory
  c0d3813 Move garbage collect code into storage package
  011b7e4 Ensure GC continues marking if _manifests is nonexistent
  0a1fcf9 Fix wording for dry-run flag in useage message for garbage collector.
  ed02e88 Sorting completed parts by part number for a better accordance with the S3 spec
  fd5a404 Add blobWrtiter.Close() call into blobWriter.Commit()
  3f538ca add cn-north-1 to valid check
  3330cc5 wait for DLO segments to show up when Close()ing the writer
  775d096 Use correct media type for config blob in schema2 manifest
  64a9727 Only check validity of S3 region if not using custom endpoint
  dafb59f Ensure we log io.Copy errors and bytes copied/total in uploads
  431e46a GCS: FileWriter.Size: return offset + buffer size for Writers that are not closed
* Thu Apr 14 2016 msabate@suse.com
- Upgraded to 2.4.0. Changelog:
  * New S3 storage driver
  The default s3 storage driver is now implemented on top of the official Amazon
  S3 SDK, boasting major performance and stability goodness. The previous storage
  is still available, but deprecated.
  * Garbage Collector
  A garbage collector tool has been added to the registry. For more details see
  the garbage collector documentation.
  * Tagged Manifest Events
  Manifest push and pull events will now include the tag which was used in the
  operation (if applicable).
  * Relative URLs
  The registry can now be configured to return relative URLs in Location headers.
  * V1 Signature disabled
  With the ongoing adoption of the schema 2 manifest format and deprecation of
  signatures, this option will improve pull performance by generating and
  returning a single libtrust signature.
  * Gotchas
  The RADOS storage driver has been removed. The registry can still be used with
  Ceph as the storage backend using the swift driver in conjunction with the
  Swift API gateway.
  The command line format has changed to support subcommands. To run a registry
  as before an additional subcommand - serve - is required.
  The legacy S3 storage driver, based on adroll/goamz is now deprecated and will
  be removed in a future release.
* Thu Mar 10 2016 msabate@suse.com
- Updated changelog
  bsc#970507
  The fix_version.patch file has been removed because it has been fixed upstream
* Thu Mar 10 2016 msabate@suse.com
- Removed old tarball
* Thu Mar 10 2016 msabate@suse.com
- Ugraded to 2.3.1. The changelog is as follows:
- Allow uppercase characters in hostnames (https://github.com/docker/distribution/commit/34c3acf8a8d800524ac06444290b26ed96d4dac0)
- Fix schema1 manifest etag and docker content digest header (https://github.com/docker/distribution/commit/d7eb5d118a6a14a6f320062296b1808506c35241)
- Add option to disable signatures (https://github.com/docker/distribution/commit/d1c173078fe47f45c7f7b96098410d4f13dd68ab)
- To avoid any network use unless necessary, delay establishing authorization (https://github.com/docker/distribution/commit/740ed699f40e1522faacb2a706e44fa1560af8ea)
- Extend authChallenger interface to remove type cast.  (https://github.com/docker/distribution/commit/16445b67679e91eab408a40a34625aa1f4dabfd1)
- Enable proxying registries to downgrade fetched manifests to Schema 1.  (https://github.com/docker/distribution/commit/36936218c2e6a4527fc99a5c04126bb1f4c7d55c)
* Fri Feb  5 2016 msabate@suse.com
- Update to 2.3
  This Docker Registry release is the first to support the Image Manifest
  Version 2, Schema 2 manifest format.
* Wed Sep 23 2015 jmassaguerpla@suse.com
- set exclusive arch to x86_64 since this is the only arch we build for
* Mon Sep  7 2015 jmassaguerpla@suse.com
- Update to 2.1.1 (bsc#948097)
  This release provides a bug fix where the filesystem layout of manifests was
  not backwards compatible with v2.0.x registries.
- Update to 2.1.0
  - Support for listing Registry repositories
  - Manifest and layer soft deletion
  - Pull through caching (experimental)
  - Storage Drivers
  more details on: https://github.com/docker/distribution/releases
  add fix_version.patch: fix version to 2.1.1 instead of git version
* Fri May 29 2015 fcastelli@suse.com
- Added README-registry.SUSE
* Fri May 29 2015 fcastelli@suse.com
- Create /etc/registry to hold all the configuration files of registry
* Thu May 14 2015 fcastelli@suse.com
- Update to 2.0.1:
  * Enable blob streaming upload
  * S3 consistency checking and better resource utilization
  * Eliminate resource leakage in the notification system
  * Request error and storage driver logging
  * Documentation corrections and additions
* Fri Apr 17 2015 fcastelli@suse.com
- Update to 2.0.0: first stable release of distribution
* Thu Apr 16 2015 fcastelli@suse.com
- Update to 2.0.0-rc4:  Several fixes have been made to contextual logging
  output, including ensuring accurate http response status, authorized user
  name and correct output of the version.
* Thu Apr  9 2015 fcastelli@suse.com
- Created initial package
